@@ -28,6 +28,7 @@
 #define HARMLESS_OK		1
 #define HARMLESS_NG		0
 #define HARMLESS_ERR		-1
+#define NO_MATCH_EXTENSION		-2
 
 #define MSG_TAG_S "<@@"
 #define MSG_TAG_E "@@>"
@@ -42,8 +43,11 @@ typedef struct _harmless_proc_arg {
     int message_length;
     char *zipdir;
     int depth;
+    int used_extension;
     struct config *cfg;
     sender_check_arg_t *maildata;
+
+    char *mime_extension; // extensionコマンドで再取得したMIMEタイプ
 } harmless_proc_arg_t;
 
 
