@@ -41,6 +41,10 @@
 #define ERR_TEMPLATE_READ		"%s: Template file is empty.(%s)"
 #define ERR_TEMPLATE_RELOADING		"%s: Cannot reload template(%s): (still refered)"
 #define TEMPLATE_RELOADING		"%s: reload template(%s)"
+#ifdef __CUSTOMIZE2018
+#define ADDMSG_RELOADING		"%s: reload add message templates"
+#define ERR_ADDMSG_RELOADING		"%s: Cannot reload add message templates: (still refered)"
+#endif	// __CUSTOMIZE2018
 
 
 /* ADD(20150316)*/
@@ -140,6 +144,7 @@
 #define ERR_NULL_VALUE		"Invalid value."
 #define ERR_DEFALT_ENCRYPTION	"Invalid default encryption strings."
 #define ERR_LDAP_FILTER		"Invalid LDAP filter strings."
+#define ERR_SET_PNOTICE         "Invalid passwordnotice value."
 
 #define ERR_LDAP_SEARCH	        "%s: Cannot search ldap.(%s):"
 #define ERR_LDAP_ENTRIE 	"%s: Cannot get first ldap entry .(variable=%s):"
