@@ -52,7 +52,11 @@ struct smfiDesc smfilter =
 {
     IDENT,
     SMFI_VERSION,   /* version code -- do not change */
+#ifndef __CUSTOMIZE2018
     SMFIF_DELRCPT,  /* flags */
+#else	// ! __CUSTOMIZE2018
+    SMFIF_DELRCPT | SMFIF_CHGHDRS,  /* flags */
+#endif	// ! __CUSTOMIZE2018
     mlfi_connect,   /* connection info filter */
     mlfi_helo,           /* SMTP HELO command filter */
     mlfi_envfrom,   /* envelope sender filter */
@@ -333,7 +337,14 @@ mlfi_envfrom(SMFICTX *ctx, char **envfrom)
         mlfi_cleanup(ctx, MLFIABORT);
 
         return SMFIS_TEMPFAIL;
-    } else if (ret == NOT_ENC) {
+#ifndef __CUSTOMIZE2018
+    if (ismode_enc) {
+        } else if (ret == NOT_ENC) {
+    }
+#else	// __CUSTOMIZE2018
+    // force into ENC mode anyway if subject enc mode
+    } else if (ret == NOT_ENC && ! priv->mlfi_conf->cff_subencmode) {
+#endif	// __CUSTOMIZE2018
         if (priv->mlfi_bccoption == DISABLE_AUTOBCC) {
             free(fromaddr);
             mlfi_cleanup(ctx, MLFIABORT);
@@ -404,6 +415,10 @@ mlfi_header(SMFICTX *ctx, char *headerf, char *headerv)
     int ret, len;
     struct mlfiPriv *priv;
     struct rcptinfo *passlist = NULL, *rdmpasslist = NULL;
+#ifdef __CUSTOMIZE2018
+    char *new_subject = NULL;
+    int chk_ret = 0;
+#endif	// __CUSTOMIZE2018
 
     if ((priv = MLFIPRIV(ctx)) == NULL) {
         log(ERR_GET_PRIV, "mlfi_header");
@@ -455,8 +470,24 @@ mlfi_header(SMFICTX *ctx, char *headerf, char *headerv)
                 }
             }
 
-            ret = search_rcptaddr(priv->mlfi_conf, priv->mlfi_savefrom,
-                    priv->mlfi_savercpt, &passlist, &rdmpasslist);
+#ifndef __CUSTOMIZE2018
+            if (ismode_enc) {
+                ret = search_rcptaddr(priv->mlfi_conf, priv->mlfi_savefrom,
+                        priv->mlfi_savercpt, &passlist, &rdmpasslist);
+            }
+#else	// ! __CUSTOMIZE2018
+            if (priv->mlfi_conf->cff_subencmode == 1) {
+                ret = search_rcptaddr(priv->mlfi_conf, priv->mlfi_savefrom,
+                        priv->mlfi_savercpt, &passlist, &rdmpasslist);
+                if (ret == NOT_ENC) {
+                    // force encryption state
+                    ret = ENC;
+                }
+            } else {
+                ret = search_rcptaddr(priv->mlfi_conf, priv->mlfi_savefrom,
+                        priv->mlfi_savercpt, &passlist, &rdmpasslist);
+            }
+#endif	// ! __CUSTOMIZE2018
         } else {
             /* Delete Mode */
             /* harmless Mode */
@@ -471,6 +502,7 @@ mlfi_header(SMFICTX *ctx, char *headerf, char *headerv)
             mlfi_cleanup(ctx, MLFIABORT);
             return SMFIS_TEMPFAIL;
         }
+
         if (ret == NOT_ENC) {
             free_rcptinfo(passlist);
             free_rcptinfo(rdmpasslist);
@@ -501,6 +533,55 @@ listvardump(passlist);
 #endif
 //DEBUGLOG("headerf= %s", headerf);
 //DEBUGLOG("headerv= %s", headerv);
+
+#ifdef __CUSTOMIZE2018
+    if (ismode_enc) {
+        if (priv->mlfi_conf->cff_subencmode == 1
+		    && strcasecmp(headerf, "Subject") == 0) {
+            // rewrite and store subject if necessary
+            if (priv->mlfi_subject == NULL) {
+                chk_ret = check_enc_subject(priv, headerv, &new_subject);
+                switch(chk_ret) {
+                case 1:	// Do encrypt
+                    priv->mlfi_subjectf = strdup(headerf);
+                    if (priv->mlfi_subjectf == NULL) {
+                        // memory error
+                        log(ERR_MEMORY_ALLOCATE, "mlfi_header", "header", strerror(errno));
+                        mlfi_cleanup(ctx, MLFIABORT);
+                        free(new_subject);
+                        return SMFIS_TEMPFAIL;
+                    }
+                    priv->mlfi_subject = new_subject;
+                    priv->mlfi_sencstatus = ENC;
+                    priv->mlfi_encstatus = ENC;
+                    headerv = new_subject;
+                    break;
+                case 0:	// Do not encrypt
+                    /* save encription status */
+                    priv->mlfi_encstatus = NOT_ENC;
+                    priv->mlfi_sencstatus = NOT_ENC;
+                    if (ismode_delete) {
+                        mlfi_cleanup(ctx, MLFIABORT);
+                        return SMFIS_ACCEPT;
+                    }
+                    if (priv->mlfi_bccoption == DISABLE_AUTOBCC) {
+                        mlfi_cleanup(ctx, MLFIABORT);
+                        return SMFIS_ACCEPT;
+                    }
+                    if (new_subject != NULL) {
+                        free(new_subject);
+                    }
+                    return SMFIS_CONTINUE;
+                default:	// Error status
+                    log(ERR_MEMORY_ALLOCATE, "mlfi_header", "header", strerror(errno));
+                    mlfi_cleanup(ctx, MLFIABORT);
+                    return SMFIS_TEMPFAIL;
+                }
+            }
+        }
+    }
+#endif	// __CUSTOMIZE2018
+
     /* create header line */
     len = strlen(headerf) + strlen(headerv) + 4;
     header = malloc(len + 1);
@@ -521,6 +602,7 @@ listvardump(passlist);
     }
     free(header);
 
+
     return SMFIS_CONTINUE;
 }
 
@@ -542,6 +624,32 @@ mlfi_eoh(SMFICTX *ctx)
         return SMFIS_TEMPFAIL;
     }
 
+#ifdef __CUSTOMIZE2018
+    if (ismode_enc) {
+        if (priv->mlfi_conf->cff_subencmode == 1
+                && priv->mlfi_sencstatus != ENC
+                && priv->mlfi_encstatus == ENC) {
+            // subject not found
+            free_rcptinfo(priv->mlfi_passlist);
+            priv->mlfi_passlist = NULL;
+            free_rcptinfo(priv->mlfi_rdmpasslist);
+            priv->mlfi_rdmpasslist = NULL;
+            /* save encription status */
+            priv->mlfi_encstatus = NOT_ENC;
+            priv->mlfi_sencstatus = NOT_ENC;
+            if (ismode_delete) {
+                mlfi_cleanup(ctx, MLFIABORT);
+                return SMFIS_ACCEPT;
+            }
+            if (priv->mlfi_bccoption == DISABLE_AUTOBCC) {
+                mlfi_cleanup(ctx, MLFIABORT);
+                return SMFIS_ACCEPT;
+            }
+            return SMFIS_CONTINUE;
+        }
+    }
+#endif	// __CUSTOMIZE2018
+
     if (priv->mlfi_encstatus == NOT_ENC) {
         return SMFIS_CONTINUE;
     }
@@ -626,6 +734,21 @@ mlfi_eom(SMFICTX *ctx)
 
     DEBUGLOG("mlfi_eom start");
 
+#ifdef __CUSTOMIZE2018
+if (ismode_enc) {
+    // Change header if EncStr exists
+    if (priv->mlfi_conf->cff_subencmode == 1
+            && priv->mlfi_sencstatus == ENC) {
+        ret = smfi_chgheader(ctx, priv->mlfi_subjectf, 1, priv->mlfi_subject);
+        if (ret != MI_SUCCESS) {
+            log("smfi_chgheader() failed. %d", ret);
+            mlfi_cleanup(ctx, MLFIABORT);
+            return SMFIS_TEMPFAIL;
+        }
+    }
+}
+#endif	// __CUSTOMIZE2018
+
     if (priv->mlfi_savebcc != NULL) {
         for (i = 0; (priv->mlfi_savebcc + i)->rcpt_addr != NULL; i++) {
             if (smfi_addrcpt(ctx, (priv->mlfi_savebcc + i)->rcpt_addr) != MI_SUCCESS) {
@@ -761,6 +884,19 @@ mlfi_cleanup(SMFICTX *ctx, int flag)
         priv->mlfi_savebcc = NULL;
     }
 
+#ifdef __CUSTOMIZE2018
+if (ismode_enc) {
+    if (priv->mlfi_subject != NULL) {
+        free(priv->mlfi_subject);
+        priv->mlfi_subject = NULL;
+    }
+    if (priv->mlfi_subjectf != NULL) {
+        free(priv->mlfi_subjectf);
+        priv->mlfi_subjectf = NULL;
+    }
+}
+#endif	// __CUSTOMIZE2018
+
     free(priv->mlfi_sendercheck_arg.ip);
     priv->mlfi_sendercheck_arg.ip = NULL;
 
@@ -1010,3 +1146,150 @@ listvardump(struct rcptinfo *passlist)
 }
 #endif
 
+#ifdef __CUSTOMIZE2018
+int
+check_enc_subject(struct mlfiPriv *priv, char *subject, char **new_subject)
+{
+    char *str_en;
+    char *str_jp;
+    int ret;
+
+    str_en = priv->mlfi_conf->cf_subjectencryptstringen;
+    str_jp = priv->mlfi_conf->cf_subjectencryptstringjp;
+
+    // search String EN
+    if (str_en != NULL && *str_en != '\0'
+        && strncmp(subject, str_en, strlen(str_en)) == 0) {
+        // found
+        *new_subject = strdup(subject + strlen(str_en));
+        if (*new_subject == NULL) {
+            return -1;
+        }
+        priv->mlfi_subject_lang = SBJLANG_EN;
+        return 1;
+    }
+
+    // search String EN from MIME part
+    ret = check_str_mime(subject, str_en, new_subject);
+    if (ret != 0) {
+        if (ret == 1) {
+            // found
+            priv->mlfi_subject_lang = SBJLANG_EN;
+        }
+        return ret;
+    }
+    
+    // search String JP
+    if (str_jp != NULL && *str_jp != '\0'
+        && strncmp(subject, str_jp, strlen(str_jp)) == 0) {
+        // found
+        *new_subject = strdup(subject + strlen(str_jp));
+        if (*new_subject == NULL) {
+            return -1;
+        }
+        priv->mlfi_subject_lang = SBJLANG_JP;
+        return 1;
+    }
+
+    // search String JP from MIME part
+    ret = check_str_mime(subject, str_jp, new_subject);
+    if (ret != 0) {
+        if (ret == 1) {
+            // found
+            priv->mlfi_subject_lang = SBJLANG_JP;
+        }
+        return ret;
+    }
+
+    // not found
+    return 0;
+}
+
+int
+check_str_mime(char *subject, char *needle, char **new_buf)
+{
+    char *dec_str, *enc_str;
+    char *start, *end;
+    char *p, *q;
+    int ret;
+    size_t n_len, new_len;
+
+    if (needle == NULL || *needle == '\0') {
+        // needle is not defined
+        return 0;
+    }
+
+    *new_buf = NULL;
+    if (strncmp(subject, "=?", 2) == 0) {
+        p = strchr(subject + 2, '?');
+        if (p != NULL && strncasecmp(p + 1, "B?", 2) == 0) {
+            start = p + 3;
+            q = strchr(start, '?');
+            if (q != NULL && *(q + 1) == '=') {
+                end = q;
+                ret = decode_b64(start, &dec_str);
+                if (ret < 0) {
+                    // memory error
+                    return -1;
+                }
+            }
+        } else if (p != NULL && strncasecmp(p + 1, "Q?", 2) == 0) {
+            start = p + 3;
+            q = strchr(start, '?');
+            if (q != NULL && *(q + 1) == '=') {
+                end = q;
+                *end = '\0';
+                ret = decode_qp(start, &dec_str);
+                *end = '?';
+                if (ret < 0) {
+                    // memory error
+                    return -1;
+                }
+            }
+        } else {
+            // not mime
+            return 0;
+        }
+    } else {
+        // not mime
+        return 0;
+    }
+
+    // case mime
+    n_len = strlen(needle);
+    if (strncmp(dec_str, needle, n_len) != 0) {
+        // not match
+        free(dec_str);
+        return 0;
+    }
+
+    // case match
+    enc_str = encode_b64(dec_str + n_len);
+    if (enc_str == NULL) {
+        // memory error
+        free(dec_str);
+        return -1;
+    }
+
+    // make new subject
+    new_len = (start - subject) + strlen(enc_str) + strlen(end);
+    *new_buf = (char *)malloc(new_len + 1);
+    if (*new_buf == NULL) {
+        // memory error
+        free(dec_str);
+        free(enc_str);
+        return -1;
+    }
+    memset(*new_buf, 0, new_len + 1);
+    strncpy(*new_buf, subject, (start - subject));
+
+    // force qp -> b64
+    *(*new_buf + (start - subject - 2)) = 'B';
+    strcat(*new_buf, enc_str);
+    strcat(*new_buf, end);
+    free(dec_str);
+    free(enc_str);
+    return 1;
+}
+
+#endif	// __CUSTOMIZE2018
