@@ -204,11 +204,14 @@ delete_attachments_mail(SMFICTX *ctx, struct mailinfo *minfo,
 	    return ZIP_CONVERT_FAILED;
 	}
 
-	/* add attach file */
-	if (add_attach_file(cfg, &mz, rdmpasslist) != 0) {
-	    mailzip_clean(mz);
-	    return ZIP_CONVERT_FAILED;
-	}
+        if ((cfg->cf_attachdeletelist[0] == 'Y') || 
+            (cfg->cf_attachdeletelist[0] == 'y')) {
+	    /* add attach file */
+	    if (add_attach_file(cfg, &mz, rdmpasslist) != 0) {
+	        mailzip_clean(mz);
+	        return ZIP_CONVERT_FAILED;
+	    }
+        }
 
         /* 送信先リストの作成 */
         for (p = rdmpasslist; p != NULL; p = p->Next) {
